#!/usr/bin/perl -w

use strict;
use Bio::SeqIO;
use Carp;
use Data::Dumper;
use English	qw ( -no_match_vars );


my $usage 							= qq (
	rewrite_fasta_defline.pl -i infile -o outfile -d defline_data_file

	takes line data from a parsed BLAST file and formats it
	for sequence deflines.
);


# open input files and output file
# parse defline data file into hash
# step thru each seq in fasta file
# 	if in hash, rewrite defline
#	if not in hash, write back out as read
# next

$OUTPUT_AUTOFLUSH 					= 1;
our %defline;

our ($infile, $outfile, $datafile);
parse_args();
parse_defline_data();

#open (my $INFILE, "<$infile")		|| die "Error opening file $infile for reading: $!\n";
#open (my $OUTFILE, ">$outfile")		|| die "Error opening file $outfile for writing: $!\n";

my $counter							= 0;

# create one SeqIO object to read in,and another to write out
my $seq_in = Bio::SeqIO->new(	'-file' 	=> "<$infile",
						  	 	'-format' 	=> 'fasta');
my $seq_out = Bio::SeqIO->new(	'-file' 	=> ">$outfile",
								'-format'	=> 'fasta');
								
# write each entry in the input file to the output file
while (my $inseq = $seq_in->next_seq()) {
	#
	$counter++;

	# get primary_id; if value exists for this, then change description
	# in all cases, write out new sequence
	my $primary_id					= $inseq->primary_id();
	if (defined $defline{$primary_id}) {
		$inseq->desc( $defline{$primary_id} . " " . $inseq->desc() )
	} else {
		$inseq->desc( "(no significant BLAST hits) " . $inseq->desc() )
	}
	$seq_out->write_seq($inseq);
	
	#exit if $counter == 1000;
}

exit;

#
#
# END OF PROGRAM
#

sub parse_args {
	#
	while (scalar(@ARGV)) {
		my $arg						= shift(@ARGV);
		if	  ($arg eq '-h') 		{ print 	$usage; } 
		elsif ($arg eq '-i') 		{ $infile 	= shift(@ARGV); }
		elsif ($arg eq '-o') 		{ $outfile 	= shift(@ARGV); }
		elsif ($arg eq '-d')		{ $datafile	= shift(@ARGV); }
		else 						{ die 		"unknown argument '$arg'"; }
	}

	# check to ensure we have the required parameters
	if (!defined $infile or !defined $outfile or !defined $datafile) {
		print "Error from undefined input arguments!\n";
		die $usage;
	}
	
}

sub parse_defline_data {
	#
	open (my $DATAFILE, "<$datafile")	|| die "Error opening file $datafile for reading: $!\n";

	while ( my $line = <$DATAFILE> ) {
		chomp $line;
		my ($key, $value) 			= split /\t/, $line;
		$defline{$key}				= $value;
	}
	close $DATAFILE;
}
